#ifndef _ERIPC_SUPPORT_
#define _ERIPC_SUPPORT_

/**
 * @addtogroup eripc
 * @{
 * @file eripc_support.h
 * @brief Public header for liberipc support functions.
 */
 
/*
 * This file is part of liberipc.
 *
 * liberipc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * liberipc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Copyright (C) 2008 iRex Technologies B.V. 
 * All rights reserved.
 *
 * Based on code found in libosso library by Kimmo Hämäläinen <kimmo.hamalainen@nokia.com>
 */

#include <glib.h>
#include "eripc.h"

G_BEGIN_DECLS

typedef struct {
    eripc_context_t *context;
    gchar *name;
    gchar *service;
    gchar *path;
} eripc_client_context_t;

typedef struct {
    gboolean    has_stylus;
    gboolean    has_wifi;
    gboolean    has_bluetooth;
    gboolean    has_3g;
} eripc_device_caps_t;

typedef struct {
    eripc_handler_t *handler;
    const gchar     *message_name;
    const gchar     *interface;
    gint            handler_id;
} eripc_callback_function_t;


/**
 * Create a new client context for a client application
 */
eripc_client_context_t *eripc_client_context_new( const char *name,
                                                  const char *version,
                                                  const char *service,
                                                  const char *path,
                                                  const char *interface,
                                                  eripc_callback_function_t *callbacks );

/**
 * Free a client context
 */
void eripc_client_context_free( eripc_client_context_t *ccontext,
                                eripc_callback_function_t *callbacks );

/**
 * Return error string for code
 */
const gchar* eripc_error_string(eripc_error_t errorcode);

/**
 * Add a new menu to the popupmenu
 */
gboolean eripc_menu_add_menu( eripc_client_context_t *ccontext,
                              const char *name,
                              const char *group1,
                              const char *group2,
                              const char *group3,
                              const char *group4 );

/**
 * Remove a menu from the popupmenu
 */
gboolean eripc_menu_remove_menu( eripc_client_context_t *ccontext,
                                 const char *name );

/**
 * Add a new group to a popupmenu menu
 */
gboolean eripc_menu_add_group( eripc_client_context_t *ccontext,
                               const char *name,
                               const char *parent, 
                               const char *image );

/**
 * Add a new item to a popupmenu group
 */
gboolean eripc_menu_add_item( eripc_client_context_t *ccontext,
                              const char *name,
                              const char *parent, 
                              const char *image  );

/**
 * Set the label for a popupmenu menu
 */
gboolean eripc_menu_set_menu_label ( eripc_client_context_t *ccontext,
                                     const char *name,
                                     const char *label );

/**
 * Set the label for a popupmenu group
 */
gboolean eripc_menu_set_group_label ( eripc_client_context_t *ccontext,
                                      const char *name,
                                      const char *label );

/**
 * Set the label for a popupmenu item
 */
gboolean eripc_menu_set_item_label ( eripc_client_context_t *ccontext,
                                     const char *name,
                                     const char *parent, 
                                     const char *label );

/**
 * Show a specific popupmenu menu
 */
gboolean eripc_menu_show_menu( eripc_client_context_t *ccontext,
                               const char *name );

/**
 * Set a popupmenu group's state
 */
gboolean eripc_menu_set_group_state( eripc_client_context_t *ccontext,
                                     const char *name,
                                     const char *state );

/**
 * Set a popupmenu item's state
 */
gboolean eripc_menu_set_item_state( eripc_client_context_t *ccontext,
                                    const char *name,
                                    const char *parent,
                                    const char *state );

/**
 * Set the statusbar page counter
 */
gboolean eripc_menu_set_pagecounter( eripc_client_context_t *ccontext,
                                     int cur_page,
                                     int num_pages,
                                     gboolean boundary_check );

/**
 * Set a statusbar item's state
 */
gboolean eripc_menu_set_statusitem_state( eripc_client_context_t *ccontext,
                                          const char *item,
                                          const char *state );

/**
 * Set the popupmenu's state
 */
gboolean eripc_sysd_set_menu_state( eripc_client_context_t *ccontext,
                                    const char *state );

/**
 * Request an SD card mount
 */
gboolean eripc_sysd_mount_sd_card( eripc_client_context_t *ccontext, gboolean show_index_splash);

/**
 * Request an SD card unmount
 */
gboolean eripc_sysd_unmount_sd_card( eripc_client_context_t *ccontext );

/**
 * Set busy state
 */
gboolean eripc_sysd_set_busy( eripc_client_context_t *ccontext,
                              const char *state,
                              const char *message );

/**
 * Reset busy state
 */
gboolean eripc_sysd_reset_busy( eripc_client_context_t *ccontext );

/**
 * Set background busy state
 */
gboolean eripc_sysd_set_bg_busy( eripc_client_context_t *ccontext );

/**
 * Reset background busy state
 */
gboolean eripc_sysd_reset_bg_busy( eripc_client_context_t *ccontext );

/**
 * Set keyboard state
 */
gboolean eripc_sysd_set_keyboard( eripc_client_context_t *ccontext, 
                                  const char *state);

/**
 * Signal the application has completed startup
 */
gboolean eripc_sysd_startup_complete( eripc_client_context_t *ccontext,
                                      const int pid,
                                      const gboolean is_multidoc,
                                      const int xid);

/**
 * Request a task be started
 */
gint eripc_sysd_start_task( eripc_client_context_t *ccontext,
                            const gchar  *cmd_line,
                            const gchar  *work_dir,
                            const gchar  *label,
                            const gchar  *thumbnail_path,
                            gchar        **err_message );

/**
 * Request a task be stopped
 */
gboolean eripc_sysd_stop_task( eripc_client_context_t *ccontext,
                              const char *cmd_line );

/**
 * Request an url be opened
 */
gint eripc_sysd_open_url ( eripc_client_context_t *ccontext,
                           const gchar  *url,
                           const gchar  *label,
                           const gchar  *back_text,
                           gchar        **err_message );

/**
 * Return the device's capabilities
 */
gboolean eripc_sysd_get_device_capabilities ( eripc_client_context_t *ccontext,
                                              eripc_device_caps_t *caps );

/**
 * Request a network connection
 */
gboolean eripc_sysd_conn_connect ( eripc_client_context_t *ccontext,
                                   const char *medium,
                                   const char *profile );
                                   
/**
 * Release the network connection
 */
gboolean eripc_sysd_conn_disconnect ( eripc_client_context_t *ccontext );

/**
 * Request the current network network status
 */
gboolean eripc_sysd_conn_status_request ( eripc_client_context_t *ccontext );

G_END_DECLS

/** @} */

#endif /* _ERIPC_ */

